"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const aws_sdk_1 = require("aws-sdk");
const errors_1 = require("./errors");
class ConnectionNotFoundError extends errors_1.ExtendableError {
}
exports.ConnectionNotFoundError = ConnectionNotFoundError;
class DynamoDBConnectionManager {
    constructor({ connectionsTable = "Connections", subscriptionsTable = "Subscriptions" } = {}) {
        this.hydrateConnection = async (connectionId) => {
            // if connection is not found, throw so we can terminate connection
            const result = await this.db
                .get({
                TableName: this.connectionsTable,
                Key: {
                    id: connectionId
                }
            })
                .promise();
            if (result.Item == null) {
                throw new ConnectionNotFoundError(`Connection ${connectionId} not found`);
            }
            return result.Item;
        };
        this.registerConnection = async ({ connectionId, endpoint }) => {
            const connection = { id: connectionId, data: { endpoint } };
            await this.db
                .put({
                TableName: this.connectionsTable,
                Item: {
                    createdAt: new Date(),
                    id: connection.id,
                    data: connection.data
                }
            })
                .promise();
            return connection;
        };
        this.sendToConnection = async (connection, payload) => {
            const { data: { endpoint }, id } = connection;
            const managementApi = new aws_sdk_1.ApiGatewayManagementApi({
                endpoint,
                apiVersion: "2018-11-29"
            });
            try {
                await managementApi.postToConnection({ ConnectionId: id, Data: payload }).promise();
            }
            catch (e) {
                // this is stale connection
                // remove it from DB
                if (e && e.statusCode === 410) {
                    await this.unregisterConnection(connection);
                }
                else {
                    throw e;
                }
            }
        };
        this.unregisterConnection = async (connection) => {
            await this.db
                .delete({
                Key: {
                    id: connection.id
                },
                TableName: this.connectionsTable
            })
                .promise();
            const subscriptions = await this.db
                .query({
                TableName: this.subscriptionsTable,
                IndexName: "connectionId_gsi",
                KeyConditionExpression: "connectionId = :connectionId",
                ExpressionAttributeValues: { ":connectionId": connection.id },
                ProjectionExpression: "event,subscriptionId"
            })
                .promise();
            const deleteRequests = subscriptions.Items.map(item => ({
                DeleteRequest: {
                    Key: {
                        event: item.event,
                        subscriptionId: item.subscriptionId
                    }
                }
            }));
            await this.db
                .batchWrite({
                RequestItems: {
                    [this.subscriptionsTable]: deleteRequests
                }
            })
                .promise();
        };
        this.connectionsTable = connectionsTable;
        this.subscriptionsTable = subscriptionsTable;
        this.db = new aws_sdk_1.DynamoDB.DocumentClient();
    }
}
exports.DynamoDBConnectionManager = DynamoDBConnectionManager;
exports.default = DynamoDBConnectionManager;
