"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const xstate_1 = require("xstate");
const { assign } = xstate_1.actions;
function connect({ backoff, handleMessage, uri, webSockImpl, }) {
    return new Promise((resolve, reject) => {
        try {
            const socket = new webSockImpl(uri);
            function onOpen() {
                // reset backoff
                backoff.reset();
                socket.onopen = null;
                socket.onerror = null;
                resolve(socket);
            }
            function onError(err) {
                socket.onopen = null;
                socket.onerror = null;
                reject(err);
            }
            socket.onerror = onError;
            socket.onopen = onOpen;
            socket.onmessage = handleMessage;
        }
        catch (e) {
            reject(e);
        }
    });
}
exports.connect = connect;
/**
 * After connection is successful assign returned socket to context
 */
function onConnectSuccess() {
    return assign((ctx, event) => (Object.assign({}, ctx, { socket: event.data })));
}
exports.onConnectSuccess = onConnectSuccess;
async function reconnect(context) {
    // wait for backoff
    const duration = context.backoff.duration();
    await new Promise(r => setTimeout(r, duration));
    return connect(context);
}
exports.reconnect = reconnect;
exports.onReconnectSuccess = onConnectSuccess;
function disconnect({ socket }) {
    return new Promise((resolve, reject) => {
        try {
            if (socket == null) {
                return resolve();
            }
            function onClose() {
                socket.onerror = null;
                socket.onclose = null;
                resolve();
            }
            function onError(err) {
                socket.onerror = null;
                socket.onclose = null;
                reject(err);
            }
            socket.onclose = onClose;
            socket.onerror = onError;
            // disconnect socket
            socket.close();
        }
        catch (e) {
            reject(e);
        }
    });
}
exports.disconnect = disconnect;
/**
 * Removes socket on successful disconnect
 */
function onDisconnectSuccess() {
    return assign(ctx => (Object.assign({}, ctx, { socket: null })));
}
exports.onDisconnectSuccess = onDisconnectSuccess;
/**
 * Processes operations (basically this runs only if client is connected)
 */
function processOperations({ operationProcessor, socket, }) {
    return (callback) => {
        // start operation processor
        operationProcessor.start(socket);
        function onClose() {
            socket.onclose = null;
            operationProcessor.stop();
            callback('DISCONNECTED'); // this will trigger reconnect or error
        }
        // register to connection close
        socket.onclose = onClose;
        return () => {
            operationProcessor.stop();
            socket.onclose = null;
        };
    };
}
exports.processOperations = processOperations;
