"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_subscriptions_1 = require("graphql-subscriptions");
const iterall_1 = require("iterall");
const ulid_1 = require("ulid");
const execute_1 = __importDefault(require("./execute"));
const formatMessage_1 = __importDefault(require("./formatMessage"));
const helpers_1 = require("./helpers");
function createWsHandler({ connectionManager, schema, subscriptionManager, }) {
    return async function serveWebSocket(event) {
        try {
            // based on routeKey, do actions
            switch (event.requestContext.routeKey) {
                case '$connect': {
                    // register connection
                    // if error is thrown during registration, connection is rejected
                    // we can implement some sort of authorization here
                    const endpoint = helpers_1.extractEndpointFromEvent(event);
                    await connectionManager.registerConnection({
                        endpoint,
                        connectionId: event.requestContext.connectionId,
                    });
                    // return will send the body to the client so we don't need to do that using
                    // connectionManager.postToConnection()
                    // you must map integration response in AWS API Gateway V2 console for this route
                    return {
                        body: formatMessage_1.default({
                            id: ulid_1.ulid(),
                            payload: {},
                            type: 'GQL_CONNECTED',
                        }),
                        statusCode: 200,
                    };
                }
                case '$disconnect': {
                    // this event is called eventually by AWS APIGateway v2
                    // we actualy don't care about a result of this operation because client is already
                    // disconnected, it is meant only for clean up purposes
                    // hydrate connection
                    const connection = await connectionManager.hydrateConnection(event.requestContext.connectionId);
                    await connectionManager.unregisterConnection(connection);
                    return;
                }
                case '$default': {
                    // here we are processing messages received from a client
                    // if we respond here and the route has integration response assigned
                    // it will send the body back to client, so it is easy to respond with operation results
                    // hydrate connection
                    const connection = await connectionManager.hydrateConnection(event.requestContext.connectionId);
                    // parse operation from body
                    const operation = helpers_1.parseOperationFromEvent(event);
                    const result = await execute_1.default({
                        connection,
                        connectionManager,
                        operation,
                        schema,
                        subscriptionManager,
                        pubSub: new graphql_subscriptions_1.PubSub(),
                        useSubscriptions: true,
                    });
                    // if result is async iterator, then it means that subscriptions was registered
                    if (iterall_1.isAsyncIterable(result)) {
                        return {
                            body: formatMessage_1.default({
                                id: operation.operationId,
                                payload: {},
                                type: 'GQL_SUBSCRIBED',
                            }),
                            statusCode: 200,
                        };
                    }
                    return {
                        body: formatMessage_1.default({
                            id: operation.operationId,
                            payload: result,
                            type: 'GQL_OP_RESULT',
                        }),
                        statusCode: 200,
                    };
                }
                default: {
                    throw new Error(`Invalid event ${event.requestContext.routeKey} received`);
                }
            }
        }
        catch (e) {
            return {
                body: e.message || 'Internal server error',
                statusCode: 500,
            };
        }
    };
}
exports.createWsHandler = createWsHandler;
exports.default = createWsHandler;
